// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysslpkcs7msg.h 2 2006-01-14 14:09:11Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A class for handling PKCS7 signatures and messages.
** 
*****************************************************************************/

#ifndef FANCYSSL_PKCS7MSG_H
#define FANCYSSL_PKCS7MSG_H

#include <fancysslbase.h>

// fwd decls from openssl.
extern "C" {
  struct pkcs7_st;
  typedef struct pkcs7_st PKCS7;
};

namespace Fancy {
  namespace SSL {
    class X509Cert;
    class PKey;

    class PKCS7Msg : public Base
    {
    public:
      PKCS7Msg ();
      PKCS7Msg (const PKCS7Msg &a);
      PKCS7Msg (PKCS7 *_pkcs7);

      ~PKCS7Msg();

      PKCS7Msg &operator= (const PKCS7Msg &a);

      bool isValid() const;
      bool isDetached() const;

      void setMessage(const std::string &m);
      void setPEMString(const char *pem, int len=-1);
      std::string getPEMString () const;

      void verify(const X509Cert &signer,
                  const X509Cert *ca_s, int n_ca_s,
                  time_t when=0);

      void sign(const PKey &singer_key,
                const X509Cert &signer,
                bool detached);

      const std::string &getMessage() const;

    private:
      PKCS7 *pkcs7;
      std::string message;
    };
  }
}

#endif
