// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysslsockdev.h 2 2006-01-14 14:09:11Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A base class, which wraps TCP/IP sockets for use with libfancy.
** Extensible for using SSL or socks etc. by using many virtual functions.
** 
*****************************************************************************/

#ifndef FANCYSSL_SOCK_DEVICE_H
#define FANCYSSL_SOCK_DEVICE_H

#include <fancysockdevice.h>
#include <fancysslbase.h>

extern "C"
{
  struct ssl_ctx_st;
  typedef struct ssl_ctx_st SSL_CTX;
};

namespace Fancy {
  namespace SSL {

    class X509Cert;
    class X509Crl;
    class PKey;

    class SockCtxPrivate;

    class SockCtx : public Base
    {
    public:
      enum Method     { SSLv2, SSLv3, SSLv23, TLSv1 };
      enum Type       { Client, Server };
      enum PeerPolicy { NoPeerCert, RequirePeerCert, VerifyPeerCert };
 
      SockCtx();

      SockCtx(const SockCtx &c);

      ~SockCtx();

      SockCtx &operator= (const SockCtx &a);

      bool isValid() const;

      void setMethod(Method m, Type t, PeerPolicy p);

      void setSessionID(const std::string &id);

      PeerPolicy getPeerPolicy() const;

      const X509Cert &myCert() const;

      void setVerifyCA(const X509Cert &ca);
      void setClientCA(const X509Cert &ca);

      void setVerifyCrl(const X509Crl &crl);

      void setCertificate(const X509Cert &x509, const PKey &pkey);

      SSL_CTX *handle() const;

    private:
      SockCtxPrivate *d;
    };

    class SockDevicePrivate;

    class SockDevice : public Fancy::Sock::Device
    {
    public:
      SockDevice(const SockCtx &ctx);

      ~SockDevice();

      const SockCtx &getSockCtx() const;

      const X509Cert &peerCert() const;

      // client stuff.
      void connect (const Fancy::Sock::Address &addr);

      // server stuff.
      Fancy::Sock::Device *accept();

      void listen (int backlog);

      void close();

      int read (void *buf, int count);
      int write (const void *buf, int count);

      void statusInfo(std::ostream &os) const;

    protected:
      SockDevice(const SockDevice &listener, const Fancy::Sock::Device::AcceptHandle &a);

      void SSLaccept();

    private:
      SockDevice(const SockDevice &);
      SockDevice &operator= (const SockDevice &);

      SockDevicePrivate *d;
    };

  }
}

#endif // ! FANCYSSL_SOCK_DEVICE_H
